import * as React from 'react';
import { Box, Button, TextField, Typography, Select, MenuItem, Stack, FormControl, InputLabel } from '@mui/material';
import { SelectChangeEvent } from '@mui/material';
import { Shipment, ContactPerson, Proposal, Address } from '../types.ts'; // Adjust import paths as necessary
import { SxProps } from '@mui/material';
import {useEffect} from "react";
import {DefaultService} from "../../openapi";

interface ShipmentFormProps {
    newShipment: Shipment;
    setNewShipment: React.Dispatch<React.SetStateAction<Shipment>>;
    handleSaveShipment: () => void;
    contactPersons: ContactPerson[];
    proposals: Proposal[];
    returnAddresses: Address[];
    sx?: SxProps;
}

const ShipmentForm: React.FC<ShipmentFormProps> = ({
                                                       newShipment,
                                                       setNewShipment,
                                                       handleSaveShipment,
                                                       contactPersons,
                                                       proposals,
                                                       returnAddresses,
                                                       sx = {},
                                                   }) => {
    const [isCreatingContactPerson, setIsCreatingContactPerson] = React.useState(false);
    const [isCreatingReturnAddress, setIsCreatingReturnAddress] = React.useState(false);
    const [newContactPerson, setNewContactPerson] = React.useState({
        firstName: '',
        lastName: '',
        phone: '',
        email: '',
    });
    const [newReturnAddress, setNewReturnAddress] = React.useState('');

    const handleContactPersonChange = (event: SelectChangeEvent) => {
        const value = event.target.value;
        if (value === 'new') {
            setIsCreatingContactPerson(true);
            setNewShipment({ ...newShipment, contact_person: [] }); // Set to empty array for new person
        } else {
            setIsCreatingContactPerson(false);
            const selectedPerson = contactPersons.find((person) => person.lastname === value) || null;
            if (selectedPerson) {
                setNewShipment({ ...newShipment, contact_person: [selectedPerson] }); // Wrap in array
            }
        }
    };

    const handleReturnAddressChange = (event: SelectChangeEvent) => {
        const value = event.target.value;
        if (value === 'new') {
            setIsCreatingReturnAddress(true);
            setNewShipment({ ...newShipment, return_address: [] }); // Set to empty array for new address
        } else {
            setIsCreatingReturnAddress(false);
            const selectedAddress = returnAddresses.find((address) => address.address === value);
            if (selectedAddress) {
                setNewShipment({ ...newShipment, return_address: [selectedAddress] }); // Wrap in array of Address
            }
        }
    };

    const handleChange = (e: React.ChangeEvent<HTMLInputElement>) => {
        const { name, value } = e.target;
        setNewShipment((prev) => ({ ...prev, [name]: value }));
    };

    const handleSaveNewContactPerson = () => {
        // Add logic to save the new contact person
        console.log('Saving new contact person:', newContactPerson);
        setIsCreatingContactPerson(false);
        setNewContactPerson({ firstName: '', lastName: '', phone: '', email: '' }); // Reset fields
    };

    const handleSaveNewReturnAddress = () => {
        // Add logic to save the new return address
        console.log('Saving new return address:', newReturnAddress);
        setIsCreatingReturnAddress(false);
        setNewReturnAddress(''); // Reset field
    };

    return (
        <Box
            sx={{
                padding: 4,
                border: '1px solid #ccc',
                borderRadius: '4px',
                marginBottom: 2,
                maxWidth: '600px',
                ...sx,
            }}
        >
            <Typography variant="h6" sx={{ marginBottom: 2 }}>
                Create Shipment
            </Typography>
            <Stack spacing={2}>
                <TextField
                    label="Shipment Name"
                    name="shipment_name"
                    value={newShipment.shipment_name || ''}
                    onChange={handleChange}
                    fullWidth
                />
                <FormControl fullWidth>
                    <InputLabel>Contact Person</InputLabel>
                    <Select
                        value={newShipment.contact_person?.[0]?.lastname || ''}
                        onChange={handleContactPersonChange}
                        displayEmpty
                    >
                        <MenuItem value="">
                            <em>Select a Contact Person</em>
                        </MenuItem>
                        {contactPersons.map((person) => (
                            <MenuItem key={person.id} value={person.lastname}>
                                {person.lastname}
                            </MenuItem>
                        ))}
                        <MenuItem value="new">
                            <em>Create New Contact Person</em>
                        </MenuItem>
                    </Select>
                </FormControl>
                {isCreatingContactPerson && (
                    <>
                        <TextField
                            label="First Name"
                            name="firstName"
                            value={newContactPerson.firstName}
                            onChange={(e) => setNewContactPerson({ ...newContactPerson, firstName: e.target.value })}
                            fullWidth
                        />
                        <TextField
                            label="Last Name"
                            name="lastName"
                            value={newContactPerson.lastName}
                            onChange={(e) => setNewContactPerson({ ...newContactPerson, lastName: e.target.value })}
                            fullWidth
                        />
                        <TextField
                            label="Phone"
                            name="phone"
                            value={newContactPerson.phone}
                            onChange={(e) => setNewContactPerson({ ...newContactPerson, phone: e.target.value })}
                            fullWidth
                        />
                        <TextField
                            label="Email"
                            name="email"
                            value={newContactPerson.email}
                            onChange={(e) => setNewContactPerson({ ...newContactPerson, email: e.target.value })}
                            fullWidth
                        />
                        <Button variant="contained" color="primary" onClick={handleSaveNewContactPerson}>
                            Save New Contact Person
                        </Button>
                    </>
                )}
                <FormControl fullWidth>
                    <InputLabel>Proposal Number</InputLabel>
                    <Select
                        value={newShipment.proposal_number || ''}
                        onChange={(e) => setNewShipment({ ...newShipment, proposal_number: e.target.value })}
                        displayEmpty
                    >
                        <MenuItem value="">
                            <em>Select a Proposal Number</em>
                        </MenuItem>
                        {proposals.map((proposal) => (
                            <MenuItem key={proposal.id} value={proposal.number}>
                                {proposal.number}
                            </MenuItem>
                        ))}
                    </Select>
                </FormControl>
                <FormControl fullWidth>
                    <InputLabel>Return Address</InputLabel>
                    <Select
                        value={newShipment.return_address?.[0]?.address || ''} // Access first return address's address
                        onChange={handleReturnAddressChange}
                        displayEmpty
                    >
                        <MenuItem value="">
                            <em>Select a Return Address</em>
                        </MenuItem>
                        {returnAddresses.map((address) => (
                            <MenuItem key={address.id} value={address.address}>
                                {address.address}
                            </MenuItem>
                        ))}
                        <MenuItem value="new">
                            <em>Create New Return Address</em>
                        </MenuItem>
                    </Select>
                </FormControl>
                {isCreatingReturnAddress && (
                    <>
                        <TextField
                            label="New Return Address"
                            value={newReturnAddress}
                            onChange={(e) => setNewReturnAddress(e.target.value)}
                            fullWidth
                        />
                        <Button variant="contained" color="primary" onClick={handleSaveNewReturnAddress}>
                            Save New Return Address
                        </Button>
                    </>
                )}
                <TextField
                    label="Comments"
                    name="comments"
                    fullWidth
                    multiline
                    rows={4}
                    value={newShipment.comments || ''}
                    onChange={handleChange}
                />
                <Button
                    variant="contained"
                    color="primary"
                    onClick={handleSaveShipment}
                    sx={{ alignSelf: 'flex-end' }}
                >
                    Save Shipment
                </Button>
            </Stack>
        </Box>
    );
};

export default ShipmentForm;
